from __future__ import absolute_import
from __future__ import unicode_literals

import time
import os
import socket

from laipvt.helper.errors import Helper
from laipvt.handler.middlewarehandler import MiddlewareConfigHandler, HarborConfigHandler
from laipvt.handler.confighandler import CheckResultHandler
from laipvt.model.cmd import ComposeModel
from laipvt.sysutil.util import path_join, ssh_obj,log
from laipvt.sysutil.gvalue import CHECK_INTERVAL
from laipvt.sysutil.template import FileTemplate
from laipvt.sysutil.command import RM, MKDIR_DIR, CHMOD_777

class MiddlewareInterface():
    def __init__(self, result: CheckResultHandler, handler: MiddlewareConfigHandler, template: str):
        self.check_result = result
        self.handler = handler
        self.template = template

        self.deploy_dir = self.check_result.deploy_dir
        self.middleware_name = self.handler.get_value("name")
        self.check_interval = CHECK_INTERVAL
        self.compose_template = path_join(self.template, "{}.tmpl".format(self.middleware_name))
        self.nginx_template = path_join(self.template, "nginx-{}.tmpl".format(self.middleware_name))
        self.servers = self.check_result.servers
        self.server_list = self.servers.get_role_ip("master")
        self.master_server = self.servers.get_role_obj("master")
        self.harbor_server = self.servers.get_role_obj("harbor")
        self.handler.set("ipaddress", self.server_list)
        self.all_server = self.servers.get()
        self.base_dir = path_join(self.deploy_dir, self.middleware_name)
        self.docker_compose_file_tmp = path_join("/tmp", "docker-compose-{}.yaml".format(self.middleware_name))
        self.nginx_config_tmp = path_join("/tmp", "nginx-{}.conf".format(self.middleware_name))

        self.docker_compose_file = path_join(self.base_dir, "docker-compose.yml")
        self.nginx_config_file = path_join(self.deploy_dir, "nginx", self.handler.get_proxy_type(), "{}.conf".format(self.middleware_name))
        self.nginx_compose_file = path_join(self.deploy_dir, "nginx", "docker-compose.yml")
        self.harbor_cfg = HarborConfigHandler().load()
        self.handler.set("harbor_http_port", self.harbor_cfg["harbor"]["http_port"])
        self.handler.set("harbor_ipaddress", [self.harbor_server[0].ipaddress, ])

    def info(self):
        return self.handler.cfg

    def wait_for_service_start(self):
        time.sleep(self.check_interval)

    def check_is_deploy(self, cfg):
        return False if cfg[self.middleware_name]["is_deploy"] == False else True

    def generate_docker_compose_file(self, data_dict) -> bool:
        FileTemplate(data_dict, self.compose_template, self.docker_compose_file_tmp).fill()
        return True if os.path.isfile(self.docker_compose_file_tmp) else False

    def send_docker_compose_file(self):
        for server in self.master_server:
            log.info(Helper().SEND_FILE.format(self.docker_compose_file_tmp, server.ipaddress, self.docker_compose_file))
            ssh_cli = ssh_obj(ip=server.ipaddress, user=server.username, password=server.password, port=server.port)
            try:
                ssh_cli.put(self.docker_compose_file_tmp, self.docker_compose_file)
            except Exception as e:
                log.error(e)
                exit(2)
            finally:
                ssh_cli.close()

    def send_docker_compose_file_hosts(self, host):
        log.info(Helper().SEND_FILE.format(self.docker_compose_file_tmp, host.ipaddress, self.docker_compose_file))
        ssh_cli = ssh_obj(ip=host.ipaddress, user=host.username, password=host.password, port=host.port)
        try:
            ssh_cli.put(self.docker_compose_file_tmp, self.docker_compose_file)
        except Exception as e:
            log.error(e)
            exit(2)
        finally:
            ssh_cli.close()

    def send_config_file(self, server, src, dest, ignore=''):
        log.info(Helper().SEND_FILE.format(src, server.ipaddress, dest))
        ssh_cli = ssh_obj(ip=server.ipaddress, user=server.username, password=server.password, port=server.port)
        try:
            ssh_cli.put(src, dest, ignore)
        except Exception as e:
            log.error(e)
            exit(2)
        finally:
            ssh_cli.close()

    def start(self):
        compose_cmd = ComposeModel(self.docker_compose_file)
        for server in self.master_server:
            log.info(Helper().START_MIDDLEWARE_SERVICE.format(server.ipaddress, self.middleware_name))
            ssh_cli = ssh_obj(ip=server.ipaddress, user=server.username, password=server.password, port=server.port)
            try:
                res = ssh_cli.run_cmd(compose_cmd.up())
                if res["code"] != 0:
                    log.error(Helper().START_MIDDLEWARE_SERVICE_ERROR.format(res["stderr"], res["stdout"]))
                    exit(2)
            except Exception as e:
                log.error(e)
                exit(2)
            finally:
                ssh_cli.close()

    def start_all_node(self):
        compose_cmd = ComposeModel(self.docker_compose_file)
        for server in self.all_server:
            log.info(Helper().START_MIDDLEWARE_SERVICE.format(server.ipaddress, self.middleware_name))
            ssh_cli = ssh_obj(ip=server.ipaddress, user=server.username, password=server.password, port=server.port)
            try:
                res = ssh_cli.run_cmd(compose_cmd.up())
                if res["code"] != 0:
                    log.error(Helper().START_MIDDLEWARE_SERVICE_ERROR.format(res["stderr"], res["stdout"]))
                    exit(2)
            except Exception as e:
                log.error(e)
                exit(2)
            finally:
                ssh_cli.close()

    def restart(self):
        compose_cmd = ComposeModel(self.docker_compose_file)
        for server in self.master_server:
            log.info(Helper().RESTART_MIDDLEWARE_SERVICE.format(server.ipaddress, self.middleware_name))
            ssh_cli = ssh_obj(ip=server.ipaddress, user=server.username, password=server.password, port=server.port)
            try:
                res = ssh_cli.run_cmd(compose_cmd.restart())
                if res["code"] != 0:
                    log.error(Helper().START_MIDDLEWARE_SERVICE_ERROR.format(res["stderr"], res["stdout"]))
                    exit(2)
            except Exception as e:
                log.error(e)
                exit(2)
            finally:
                ssh_cli.close()

    def reset(self):
        compose_cmd = ComposeModel(self.docker_compose_file)
        for server in self.master_server:
            log.info(Helper().RESET_MIDDLEWARE_SERVICE.format(server.ipaddress, self.middleware_name))
            ssh_cli = ssh_obj(ip=server.ipaddress, user=server.username, password=server.password, port=server.port)
            try:
                res = ssh_cli.run_cmd(compose_cmd.down())
                res = ssh_cli.run_cmd(compose_cmd.up())
                if res["code"] != 0:
                    log.error(Helper().START_MIDDLEWARE_SERVICE_ERROR.format(res["stderr"], res["stdout"]))
                    exit(2)
            except Exception as e:
                log.error(e)
                exit(2)
            finally:
                ssh_cli.close()

    def create_logs_dir(self, log_path):
        log.info(Helper().CREATE_LOG_PATH.format(log_path))
        cmd = [
            MKDIR_DIR.format(log_path),
            CHMOD_777.format(log_path)
        ]
        for server in self.master_server:
            ssh_cli = ssh_obj(ip=server.ipaddress, user=server.username,
                              password=server.password, port=server.port)
            ssh_cli.run_cmdlist(cmd)

    def check(self) -> bool:
        pass

    def check_port(self, ip, port):
        try:
            if int(port) > 65536:
                return False
            else:
                s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
                result = s.connect_ex((ip, port))
                if result == 0:
                    return True
                s.close()
        except Exception as e:
            log.error(e)
            return False

    def init(self, path) -> bool:
        FileTemplate(self.handler.cfg, self.nginx_template, self.nginx_config_tmp).fill()
        return True if os.path.isfile(self.nginx_config_tmp) else False

    def update_nginx_config(self):
        compose_cmd = ComposeModel(self.nginx_compose_file)
        for server in self.master_server:
            ssh_cli = ssh_obj(ip=server.ipaddress, user=server.username, password=server.password, port=server.port)
            try:
                ssh_cli.put(self.nginx_config_tmp, self.nginx_config_file)
                ssh_cli.run_cmd(compose_cmd.restart())
            except Exception as e:
                log.error(e)
                exit(2)
            finally:
                ssh_cli.close()

    def remove(self):
        compose_cmd = ComposeModel(self.docker_compose_file)
        for server in self.master_server:
            ssh_cli = ssh_obj(ip=server.ipaddress, user=server.username, password=server.password, port=server.port)
            try:
                ssh_cli.run_cmd(compose_cmd.down())
                ssh_cli.run_cmd(RM.format(self.base_dir))
            except Exception as e:
                log.error(e)
                exit(2)
            finally:
                ssh_cli.close()

    def run(self):
        self.generate_docker_compose_file()
        self.send_docker_compose_file()
        self.start()
        self.check()
        self.update_nginx_config()



from __future__ import absolute_import
from __future__ import unicode_literals

import re
import time
import requests
from requests.auth import HTTPBasicAuth
from laipvt.interface.middlewareinterface import MiddlewareInterface
from laipvt.handler.middlewarehandler import EsConfigHandler
from laipvt.handler.confighandler import CheckResultHandler
from laipvt.sysutil.template import FileTemplate
from laipvt.sysutil.util import path_join, log, status_me
from laipvt.helper.errors import Helper


class EsController(MiddlewareInterface):
    def __init__(self, result: CheckResultHandler, handler: EsConfigHandler, template: str):
        super(EsController, self).__init__(result, handler, template)
        self.es_conf_tmp = path_join("/tmp", "elasticsearch.yml")
        self.es_conf_template = path_join(self.template, "config.tmpl")
        self.es_conf_file = path_join(self.base_dir, "conf", "elasticsearch.yml")
        self.auth_conf_tmp = path_join("/tmp", "readonlyrest.yml")
        self.auth_conf_template = path_join(self.template, "readonlyrest.yml")
        self.auth_conf_file = path_join(self.base_dir, "conf", "readonlyrest.yml")
        self.pligins = path_join(self.template, "plugins")
        self.pligins_tmp = path_join(self.base_dir, "plugins")
        self.es_cfg = EsConfigHandler().get_config_with_check_result()
        self.http_port = self.es_cfg["elasticsearch"]["http_port"]
        self.password = self.es_cfg["elasticsearch"]["password"]
        self.es_cfg["elasticsearch"]["ipaddress"] = self.handler.cfg["ipaddress"]

    def _generic_config(self):
        for num_id in range(len(self.master_server)):
            self.es_cfg["elasticsearch"]["host_name"] = "saas-es-0%s" % (num_id + 1)
            self.es_cfg["elasticsearch"]["master_node"] = "saas-es-01"
            self.es_cfg["ip"] = self.master_server[num_id].ipaddress

            FileTemplate(self.es_cfg, self.es_conf_template, self.es_conf_tmp).fill()
            FileTemplate(self.es_cfg, self.auth_conf_template, self.auth_conf_tmp).fill()

            self.send_config_file(self.master_server[num_id], self.es_conf_tmp, self.es_conf_file)
            self.send_config_file(self.master_server[num_id], self.auth_conf_tmp, self.auth_conf_file)
            self.send_config_file(self.master_server[num_id], self.pligins, self.pligins_tmp)
            self.generate_docker_compose_file(self.es_cfg)

    def _check(self):
        time.sleep(60)
        for i in self.master_server:
            try:
                log.info(Helper().CHECK_MIDDLEWARE_SERVICE.format(self.middleware_name, i.ipaddress, self.http_port))
                url = "http://{IP}:{PORT}/_cat/health".format(
                    IP=i.ipaddress, PORT=self.http_port
                )
                result = requests.get(url,  auth=HTTPBasicAuth('admin', '{}'.format(
                    self.password))).content.decode("utf-8")

                if re.search("green", result):
                    # log.info("Elasticsearch 检查通过")
                    self._read_write_test(i.ipaddress, self.http_port)
                else:
                    log.info(Helper().CHECK_MIDDLEWARE_SERVICE_PORT_FAILED.format(i.ipaddress, self.http_port))
                    log.error(result)
                    exit(2)
            except Exception as e:
                log.info(Helper().CHECK_MIDDLEWARE_SERVICE_PORT_FAILED.format(i.ipaddress, self.http_port))
                log.error(e)
                exit(2)

    def _read_write_test(self, ipaddress, port):
        try:
            headers = {"Content-Type": "application/json"}
            create_index = "http://{IP}:{PORT}/laiye_test/_doc/1".format(IP=ipaddress, PORT=port)
            delete_index = "http://{IP}:{PORT}/laiye_test".format(IP=ipaddress, PORT=port)
            data_dit = {"name": "laiye", "age": "5"}
            auth = HTTPBasicAuth('admin', '{}'.format(self.password))

            requests.post(url=create_index, json=data_dit,  headers=headers, auth=auth)
            # log.info("Elasticsearch 测试数据插入成功")
            log.info(Helper().CHECK_MIDDLEWARE_SERVICE_PORT_SUCCEED.format(ipaddress, port))
            requests.delete(url=delete_index, auth=auth)
        except Exception as e:
            log.error(e)
            exit(2)

    @status_me("middleware")
    def deploy_es(self):
        if self.check_is_deploy(self.es_cfg):
            self._generic_config()
            self.send_docker_compose_file()
            self.start()
            self._check()

    def deploy(self):
        self.deploy_es()



from __future__ import absolute_import
from __future__ import unicode_literals

class LaiyeException(Exception):
    def __init__(self, msg = "error", code="128"):
        self.msg = msg
        self.code = code

class SSHAuthFailed(Exception):
    def __init__(self, *args, **kwargs):
        self.msg = "ssh账号密码错误"

class SSHConnectFailed(Exception):
    def __init__(self, *args, **kwargs):
        self.msg = "ssh连接失败"

class SSHPortFailed(Exception):
    def __init__(self, *args, **kwargs):
        self.msg = "ssh端口错误"

class SSHAuthInfoMissed(Exception):
    def __init__(self, *args, **kwargs):
        self.msg = "未找到ssh密码或密钥"

class SSHPortOrIpIllegal(Exception):
    def __init__(self, *args, **kwargs):
        self.msg = "IP地址或端口号不合法"

class UtilsError(LaiyeException):
    pass

class ModelError(LaiyeException):
    pass

class HandlerError(LaiyeException):
    pass

class Forbidden(LaiyeException):
    pass


from __future__ import absolute_import
from __future__ import unicode_literals

from laipvt.handler.confighandler import CheckResultHandler
from laipvt.handler.middlewarehandler import HarborConfigHandler
from laipvt.sysutil.util import path_join


class KubeConfigHandler(object):
    def __init__(self, check_result: CheckResultHandler, *args, **kwargs):
        """
        初始化k8s的配置类
        :param check_result:
        :param args:
        :param kwargs:  subnet, version, network_plugin
        """
        # self.first_master = check_result.servers.get_role_ip("master")[0]
        self.first_master = check_result.servers.get_role_ip("harbor")[0]

        self.apiserver_porxy_port = "6443"
        self.apiserver_proxy_address = "127.0.0.1"
        self.lb_proxy_port = "6444"
        self.version = kwargs.get("version", "v1.19.16")

        # self.harbor_endpoint = "{}:{}".format(check_result.servers.get_role_ip("harbor")[0], 8888)
        self.harbor_cfg = HarborConfigHandler().get_config_with_check_result()
        self.harbor_endpoint = "{}:{}".format(self.harbor_cfg["harbor"]["lb"], self.harbor_cfg["harbor"]["nginx_harbor_proxy_port"])
        self.image_repo = path_join(self.harbor_endpoint, "base_system")
        self.subnet = kwargs.get("subnet", "10.244.0.0/16")

        self.ip_list = []
        self.ip_list.append(check_result.lb)

        self.harbor_list = check_result.servers.get_role_ip("harbor")
        self.master_list = check_result.servers.get_role_ip("master")
        self.node_list = check_result.servers.get_role_ip("node")
        self.others_master_list = sorted(list(set(self.master_list) ^ set(self.harbor_list)))

        self.ip_list += self.master_list
        self.ip_list += self.node_list
        self.ip_list = list(set(self.ip_list))

        self.network_plugin = kwargs.get("network_plugin", "flannel")

    def get(self) -> dict:
        return self.__dict__


"""
# cat admin.tmpl
apiVersion: kubeadm.k8s.io/v1beta2
kind: InitConfiguration
localAPIEndpoint:
  advertiseAddress: {{ first_master }}
  bindPort: {{ apiserver_porxy_port }}
nodeRegistration:
  criSocket: /var/run/dockershim.sock
  name: master-01
  taints:
  - effect: NoSchedule
    key: node-role.kubernetes.io/master
---
apiServer:
  certSANs:
    - 127.0.0.1 {% for key in ip_list %}
    - {{ key }}{% endfor %}
  timeoutForControlPlane: 4m0s
apiVersion: kubeadm.k8s.io/v1beta2
certificatesDir: /etc/kubernetes/pki
clusterName: kubernetes
controlPlaneEndpoint: "{{ apiserver_proxy_address }}:{{ nginx_proxy_port }}"

controllerManager: {}
dns:
  type: CoreDNS
etcd:
  local:
    dataDir: /var/lib/etcd
imageRepository: {{ image_repo }}
kind: ClusterConfiguration
kubernetesVersion: v1.18.12
networking:
  dnsDomain: cluster.local
  serviceSubnet: 10.96.0.0/12
  podSubnet: "{{ subnet }}"
scheduler: {}
---
apiVersion: kubeproxy.config.k8s.io/v1alpha1
kind: KubeProxyConfiguration
mode: ipvs
"""